package com.reactlibraryneurosdk;

import androidx.annotation.NonNull;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.neuromd.neurosdk.ChannelInfo;
import com.neuromd.neurosdk.ChannelType;
import com.neuromd.neurosdk.Device;
import com.neuromd.neurosdk.ResistanceChannel;
import com.neuromd.neurosdk.SignalChannel;

import java.util.HashMap;

public class BatteryChannel {

    private com.neuromd.neurosdk.BatteryChannel channelBattery;

    public com.neuromd.neurosdk.BatteryChannel getChannelBattery() {
        return channelBattery;
    }

    public BatteryChannel(@NonNull ReactApplicationContext reactContext, @NonNull Device device, Promise promise){
        try{
            channelBattery = new com.neuromd.neurosdk.BatteryChannel(device);
            channelBattery.dataLengthChanged.subscribe((channel, length) -> {
                WritableMap resultMap = Arguments.createMap();
                resultMap.putString("name", channelBattery.info().getName());
                resultMap.putInt("index", (int)channelBattery.info().getIndex());
                resultMap.putString("type", channelBattery.info().getType().toString());
                resultMap.putInt("length", length);
                Utils.sendEvent(reactContext, RNNeurosdkModule.BATTERY_CHANNEL_LENGTH_CHANGED_EVENT_NAME, resultMap);
            });
            WritableMap resultState = Arguments.createMap();
            resultState.putString("State","Created");
            promise.resolve(resultState);
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }

    public void TotalLength(Promise promise){
        try{
            if(channelBattery != null){
                WritableMap result = Arguments.createMap();
                result.putInt("totalLength", channelBattery.totalLength());
                promise.resolve(result);
            } else {
                promise.reject("Error","Battery channel doesnt exist");
            }
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }
    public WritableMap TotalLengthSynchronous(){
        WritableMap result = Arguments.createMap();
        result.putInt("totalLength", -1);
        try{
            if(channelBattery != null){
                result.putInt("totalLength", channelBattery.totalLength());
                return result;
            }
        }
        catch(Exception ex){}
        finally {
            return result;
        }
    }
    public void ReadData(int offset, int length, Promise promise){
        try{
            if(channelBattery != null){
                int[] signalData = channelBattery.readData(offset, length);
                WritableArray result = Arguments.createArray();
                for (int sample:
                        signalData) {
                    result.pushInt(sample);
                }
                promise.resolve(result);
            } else
            {
                promise.reject("Error","Battery channel doesnt exist");
            }
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }
    public WritableArray ReadDataSynchronous(int offset, int length){
        int[] signalData = channelBattery.readData(offset, length);
        WritableArray result = Arguments.createArray();
        for (int sample:
                signalData) {
            result.pushInt(sample);
        }
        return result;
    }

    public void CloseChannel( Promise promise){
        try{
            if(channelBattery != null){
                if(channelBattery.dataLengthChanged != null){
                    channelBattery.dataLengthChanged.unsubscribe();
                }
                channelBattery = null;
                WritableMap result = Arguments.createMap();
                result.putNull("Success");
                promise.resolve(result);
            } else
            {
                promise.reject("Error","Battery channel doesnt exist");
            }
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }
}
