package com.reactlibraryneurosdk;

import androidx.annotation.NonNull;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.neuromd.neurosdk.ChannelInfo;
import com.neuromd.neurosdk.ChannelType;
import com.neuromd.neurosdk.Device;

public class ResistanceChannel {
    private com.neuromd.neurosdk.ResistanceChannel resistanceChannel;
    public com.neuromd.neurosdk.ResistanceChannel getResistanceChannel() {
        return resistanceChannel;
    }
    public ResistanceChannel(@NonNull ReactApplicationContext reactContext,@NonNull ChannelInfo channelInfo, @NonNull Device device, Promise promise) {
        try{
            resistanceChannel = new com.neuromd.neurosdk.ResistanceChannel(device, channelInfo);
            resistanceChannel.dataLengthChanged.subscribe((channel, length) -> {
                WritableMap resultMap = Arguments.createMap();
                resultMap.putString("name", resistanceChannel.info().getName());
                resultMap.putInt("index", (int)resistanceChannel.info().getIndex());
                resultMap.putString("type", resistanceChannel.info().getType().toString());
                resultMap.putInt("length", length);
                Utils.sendEvent(reactContext, RNNeurosdkModule.RESISTANCE_CHANNEL_LENGTH_CHANGED_EVENT_NAME, resultMap);
            });
            WritableMap resultState = Arguments.createMap();
            resultState.putString("State","Created");
            promise.resolve(resultState);
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }

    public void TotalLength(Promise promise){
        try{
            WritableMap result = Arguments.createMap();
            result.putInt("totalLength", resistanceChannel.totalLength());
            promise.resolve(result);
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }
    public WritableMap TotalLengthSynchronous(){
        WritableMap result = Arguments.createMap();
        result.putInt("totalLength", -1);
        try{
            if(resistanceChannel != null){
                result.putInt("totalLength", resistanceChannel.totalLength());
                return result;
            }
        }
        catch(Exception ex){}
        finally {
            return result;
        }
    }
    public void ReadData(int offset, int length, Promise promise){
        try{
            double[] resistanceData = resistanceChannel.readData(offset, length);
            WritableArray result = Arguments.createArray();
            for (double sample:
                    resistanceData) {
                result.pushDouble(sample);
            }
            promise.resolve(result);
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }
    public WritableArray ReadDataSynchronous(int offset, int length){
        double[] resistanceData = resistanceChannel.readData(offset, length);
        WritableArray result = Arguments.createArray();
        for (double sample:
                resistanceData) {
            result.pushDouble(sample);
        }
        return result;
    }
    public void CloseChannel(Promise promise){
        try{
            if(resistanceChannel.dataLengthChanged != null){
                resistanceChannel.dataLengthChanged.unsubscribe();
            }
            resistanceChannel = null;
            WritableMap result = Arguments.createMap();
            result.putNull("Success");
            promise.resolve(result);
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }
}
