package com.reactlibraryneurosdk;

import androidx.annotation.NonNull;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.neuromd.neurosdk.ChannelInfo;
import com.neuromd.neurosdk.ChannelType;
import com.neuromd.neurosdk.Device;

public class SignalChannel {

    private com.neuromd.neurosdk.SignalChannel signalChannel;

    public com.neuromd.neurosdk.SignalChannel getSignalChannel() {
        return signalChannel;
    }

    public SignalChannel(@NonNull ReactApplicationContext reactContext,@NonNull ChannelInfo channelInfo, @NonNull Device device, Promise promise) {
        try{
            signalChannel = new com.neuromd.neurosdk.SignalChannel(device,channelInfo);
            signalChannel.dataLengthChanged.subscribe((channel, length) -> {
                WritableMap resultMap = Arguments.createMap();
                resultMap.putString("name", signalChannel.info().getName());
                resultMap.putInt("index", (int)signalChannel.info().getIndex());
                resultMap.putString("type", signalChannel.info().getType().toString());
                resultMap.putInt("length", length);
                Utils.sendEvent(reactContext, RNNeurosdkModule.SIGNAL_CHANNEL_LENGTH_CHANGED_EVENT_NAME, resultMap);
            });
            WritableMap resultState = Arguments.createMap();
            resultState.putString("State","Created");
            promise.resolve(resultState);
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }

    public void TotalLength(Promise promise){
        try{
            WritableMap result = Arguments.createMap();
            result.putInt("totalLength", signalChannel.totalLength());
            promise.resolve(result);
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }

    public WritableMap TotalLengthSynchronous(){
        WritableMap result = Arguments.createMap();
        result.putInt("totalLength", -1);
        try{
            if(signalChannel != null){
                result.putInt("totalLength", signalChannel.totalLength());
                return result;
            }
        }
        catch(Exception ex){}
        finally {
            return result;
        }
    }

    public void ReadData(int offset, int length, Promise promise){
        try{
            double[] signalData = signalChannel.readData(offset, length);
            WritableArray result = Arguments.createArray();
            for (double sample : signalData) {
                result.pushDouble(sample);
            }
            promise.resolve(result);
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }
    public WritableArray ReadDataSynchronous(int offset, int length){
        double[] signalData = signalChannel.readData(offset, length);
        WritableArray result = Arguments.createArray();
        for (double sample : signalData) {
            result.pushDouble(sample);
        }
        return result;
    }
    public void CloseChannel( Promise promise){
        try{
            if(signalChannel.dataLengthChanged != null){
                signalChannel.dataLengthChanged.unsubscribe();
            }
            signalChannel = null;
            WritableMap result = Arguments.createMap();
            result.putNull("Success");
            promise.resolve(result);
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }
}
