//
//  BrainBit.h
//  RNNeurosdk
//
//  Created by Aseatari on 23.05.2022.
//  Copyright © 2022 Facebook. All rights reserved.
//

#ifndef BrainBit_h
#define BrainBit_h

#import <neurosdk/neurosdk.h>

@protocol BrainBitDelegate <NSObject>
- (void) onStateChange: (NSString*)address state:(NTState)state;
- (void) onLengthChanged:(NTChannelType)type info:(NTChannelInfo*)info length:(NSUInteger) length;
@end

@interface BrainBit : NSObject
{
    NTDevice* _device;
    
    NTBatteryChannel* _batteryChannel;
    
    NSMutableDictionary<NSString*, NTResistanceChannel*>* _resistanceChannels;
    NSMutableDictionary<NSString*, NTSignalChannel*>* _signalChannels;
}

@property(nonatomic, strong) id<BrainBitDelegate> brainbitDelegate;

- (void) createDevice: (NTDeviceEnumerator*)deviceEnumerator deviceInfo:(NTDeviceInfo*)devInfo;
- (void) connect;
- (void) disconnect;
- (void) subscribeToParameterChanged;
- (void) unsubscribeToParameterChanged;
- (void) close;

- (NTFirmwareVersion*) version;
- (NSArray*) channels;
- (NSArray*) commands;
- (NSArray*) parameters;

- (void) initBattery;
- (NSArray<NSNumber*>*) readBatteryData: (NSUInteger)offset length:(NSUInteger) length;
- (NSUInteger) batteryTotalLength;
- (void) destroyBattery;

- (void) startResist;
- (void) stopResist;

- (void) initResistanceChannelWithName:(NSString*)channelName;
- (NSArray<NSNumber*>*) readResistanceDataFromChannel: (NSString*)channelName offset:(NSUInteger)offset length:(NSUInteger) length;
- (NSUInteger) resistanceTotalLengthOfChannel: (NSString*)channelName;
- (void) destroyResistanceWithName:(NSString*)channelName;

- (void) startSignal;
- (void) stopSignal;

- (void) initSignalWithName:(NSString*)channelName;
- (NSArray<NSNumber*>*) readSignalDataFromChannel: (NSString*)channelName offset:(NSUInteger)offset length:(NSUInteger) length;
- (NSUInteger) signalTotalLengthOfChannel: (NSString*)channelName;
- (void) destroySignalWithName:(NSString*)channelName;

            
@end

#endif /* BrainBit_h */
