
#import "RNNeurosdk.h"
@interface RNNeurosdk()

- (void) createEnumerator;
- (void) closeEnumerator;


@end

@implementation RNNeurosdk
{
    NTDeviceEnumerator* _deviceEnumerator;
    NSMutableDictionary<NSString*, BrainBit*>* _devices;
    NSMutableArray<NTDeviceInfo*>* foundedDevices;
    bool hasListeners;
    
}

NSString* DEVICE_LIST_CHANGED_EVENT_NAME = @"DeviceListChanged";
NSString* DEVICE_PARAMETER_CHANGED_EVENT_NAME = @"DeviceParameterChanged";
NSString* SIGNAL_CHANNEL_LENGTH_CHANGED_EVENT_NAME = @"SignalChannelLengthChanged";
NSString* RESISTANCE_CHANNEL_LENGTH_CHANGED_EVENT_NAME = @"ResistanceChannelLengthChanged";
NSString* BATTERY_CHANNEL_LENGTH_CHANGED_EVENT_NAME = @"BatteryChannelLengthChanged";

NSString* BATTERY_CHANNEL = @"Battery";
NSString* SIGNAL_CHANNEL = @"Signal";
NSString* RESISTANCE_CHANNEL = @"Resistance";

NSString* START_SIGNAL = @"StartSignal";
NSString* STOP_SIGNAL = @"StopSignal";
NSString* START_RESIST = @"StartResist";
NSString* STOP_RESIST = @"StopResist";



- (instancetype)init {
  self = [super init];
  if (self) {
      foundedDevices = [NSMutableArray new];
      _devices = [NSMutableDictionary new];
  }
  return self;
}

+ (BOOL)requiresMainQueueSetup
{
  return NO;
}

-(void)startObserving{
    hasListeners = YES;
}

-(void)stopObserving{
    hasListeners = NO;
}

- (NSDictionary*) constantsToExport{
    return @{ @"DEVICE_LIST_CHANGED_EVENT_NAME": DEVICE_LIST_CHANGED_EVENT_NAME,
              @"DEVICE_PARAMETER_CHANGED_EVENT_NAME": DEVICE_PARAMETER_CHANGED_EVENT_NAME,
              @"SIGNAL_CHANNEL_LENGTH_CHANGED_EVENT_NAME":SIGNAL_CHANNEL_LENGTH_CHANGED_EVENT_NAME,
              @"RESISTANCE_CHANNEL_LENGTH_CHANGED_EVENT_NAME":RESISTANCE_CHANNEL_LENGTH_CHANGED_EVENT_NAME,
              @"BATTERY_CHANNEL_LENGTH_CHANGED_EVENT_NAME":BATTERY_CHANNEL_LENGTH_CHANGED_EVENT_NAME,
              @"BATTERY_CHANNEL": BATTERY_CHANNEL,
              @"SIGNAL_CHANNEL":SIGNAL_CHANNEL,
              @"RESISTANCE_CHANNEL":RESISTANCE_CHANNEL,
              @"START_SIGNAL":START_SIGNAL,
              @"STOP_SIGNAL":STOP_SIGNAL,
              @"START_RESIST":START_RESIST,
              @"STOP_RESIST":STOP_RESIST
    };
}

RCT_EXPORT_MODULE()


-(NSArray<NSString*>*) supportedEvents{
    return @[DEVICE_LIST_CHANGED_EVENT_NAME,
             DEVICE_PARAMETER_CHANGED_EVENT_NAME,
             SIGNAL_CHANNEL_LENGTH_CHANGED_EVENT_NAME,
             RESISTANCE_CHANNEL_LENGTH_CHANGED_EVENT_NAME,
             BATTERY_CHANNEL_LENGTH_CHANGED_EVENT_NAME];
}

-(void) neuroSendEvent:(NSString*)name params:(NSDictionary*)params{
    if(hasListeners){
        [self sendEventWithName:name body:params];
    }
}

- (void) onStateChange:(NSString*)address state: (NTState)state{
    NSString* nameConnectionString = state == NTStateConnected? @"Connected" : @"Disconnected";
    NSDictionary* currentState = [NSDictionary dictionaryWithObjectsAndKeys:address, @"address", nameConnectionString, @"state", nil];
    [self neuroSendEvent:DEVICE_PARAMETER_CHANGED_EVENT_NAME params:currentState];
}

- (void) onLengthChanged:(NTChannelType)type info:(NTChannelInfo*)info length:(NSUInteger) length{
    NSMutableDictionary* result = [NSMutableDictionary new];
    if(type == NTChannelTypeBattery){
        [result setObject:BATTERY_CHANNEL forKey:@"name"];
        [result setObject:@"0" forKey:@"index"];
        [result setObject:BATTERY_CHANNEL forKey:@"type"];
        [result setObject:[NSString stringWithFormat:@"%lu", (unsigned long)length] forKey:@"length"];
    } else {
        NSString* channelType = @"";
            switch (type) {
                case NTChannelTypeSignal:
                    channelType = SIGNAL_CHANNEL;
                    break;
                case NTChannelTypeBattery:
                    channelType = BATTERY_CHANNEL;
                    break;
                case NTChannelTypeResistance:
                    channelType = RESISTANCE_CHANNEL;
                    break;
                default:
                    break;
            }
        [result setObject:info.name forKey:@"name"];
        [result setObject:[NSString stringWithFormat:@"%d", (int)info.index] forKey:@"index"];
        [result setObject:channelType forKey:@"type"];
        [result setObject:[NSString stringWithFormat:@"%lu", (unsigned long)length] forKey:@"length"];
    }
    
    NSString* eventName = @"";
    switch (type) {
        case NTChannelTypeSignal:
            eventName = SIGNAL_CHANNEL_LENGTH_CHANGED_EVENT_NAME;
            break;
        case NTChannelTypeResistance:
            eventName = RESISTANCE_CHANNEL_LENGTH_CHANGED_EVENT_NAME;
            break;
        case NTChannelTypeBattery:
            eventName = BATTERY_CHANNEL_LENGTH_CHANGED_EVENT_NAME;
            break;
        default:
            break;
    }
    [self neuroSendEvent:eventName params:result];
}

RCT_EXPORT_METHOD(CreateDeviceEnumerator:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject)
{
    [self createEnumerator];
    if(_deviceEnumerator){
        resolve(@"Device enumerator created");
    } else {
        reject(@"Error", @"Fail create device enumerator", nil);
    }
}

RCT_EXPORT_METHOD(Devices:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject)
{
    if([foundedDevices count] > 0){
        NSMutableArray<NSDictionary*>* result = [NSMutableArray new];
        for (NTDeviceInfo* di in foundedDevices) {
            NSString* sn = [[NSNumber numberWithUnsignedLongLong:[di serialNumber]] stringValue];
            NSDictionary<NSString*, NSString*>* info = [[NSDictionary alloc] initWithObjectsAndKeys:[di name], @"name", [di address], @"address", sn, @"serialNumber", nil];
            [result addObject:info];
        }
        resolve(result);
    } else {
        reject(@"Error", @"Devices not found", nil);
    }
}

RCT_EXPORT_METHOD(CreateDevice:(NSDictionary*) deviceInfo
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject)
{
    NSString* address = [deviceInfo objectForKey:@"address"];
    if(address){
        NTDeviceInfo* devInfo = [[NTDeviceInfo alloc] initWithName:deviceInfo[@"name"] address:deviceInfo[@"address"] serialNumber:[deviceInfo[@"serialNumber"] integerValue]];
        BrainBit* brainbit = [BrainBit new];
        [brainbit createDevice:_deviceEnumerator deviceInfo:devInfo];
        [_devices setObject:brainbit forKey:deviceInfo[@"address"]];
        resolve(@"Device created");
    }else{
        reject(@"Device create failed: ", @"Device info null", nil);
    }
}

RCT_EXPORT_METHOD(DeleteDeviceEnumerator)
{
    [self closeEnumerator];
}

- (void) createEnumerator{
    [self closeEnumerator];
    [foundedDevices removeAllObjects];
    _deviceEnumerator = [[NTDeviceEnumerator alloc] initWithDeviceType:TypeBrainbitAny];
    [_deviceEnumerator subscribeFoundDeviceWithSubscriber:^(NTDeviceInfo* deviceInfo){
        NSPredicate *predicate = [NSPredicate predicateWithFormat:@"address == %@", deviceInfo.address];
        NSArray *filteredArray = [self->foundedDevices filteredArrayUsingPredicate:predicate];
        if([filteredArray count] < 1){
            [self->foundedDevices addObject:deviceInfo];
            [self neuroSendEvent:DEVICE_LIST_CHANGED_EVENT_NAME params:nil];
        }
    }];
}
 
- (void) closeEnumerator{
    if(_deviceEnumerator){
        [_deviceEnumerator subscribeFoundDeviceWithSubscriber:nil];
    }
    _deviceEnumerator = nil;
}

/* Device */
RCT_EXPORT_METHOD(BoundTo:(NSString*)address
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject){
    resolve(@"Bonded");
}

RCT_EXPORT_METHOD(Connect:(NSDictionary*) deviceInfo
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject)
{
    BrainBit* device = [_devices objectForKey:deviceInfo[@"address"]];
    if(device == nil){
        reject(@"Error: ", @"Deviced is null", nil);
        return;
    }
    device.brainbitDelegate = self;
    [device connect];
    NSDictionary* currentState = [NSDictionary dictionaryWithObjectsAndKeys:@"Connection", @"state", nil];
    resolve(currentState);
}

RCT_EXPORT_METHOD(SubscribeParameterChanged:(NSDictionary*) deviceInfo
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject)
{
    BrainBit* device = [_devices objectForKey:deviceInfo[@"address"]];
    if(device == nil){
        reject(@"Error: ", @"Deviced is null", nil);
        return;
    }
    [device subscribeToParameterChanged];
    NSDictionary* currentState = [NSDictionary dictionaryWithObjectsAndKeys:@"Subscription", @"successful", nil];
    resolve(currentState);
}

RCT_EXPORT_METHOD(UnsubscribeParameterChanged:(NSDictionary*) deviceInfo
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject)
{
    BrainBit* device = [_devices objectForKey:deviceInfo[@"address"]];
    if(device == nil){
        reject(@"Error: ", @"Deviced is null", nil);
        return;
    }
    [device unsubscribeToParameterChanged];
    NSDictionary* currentState = [NSDictionary dictionaryWithObjectsAndKeys:@"Unubscription", @"successful", nil];
    resolve(currentState);
}

RCT_EXPORT_METHOD(FirmwareVersion:(NSDictionary*) deviceInfo
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject)
{
    BrainBit* device = [_devices objectForKey:deviceInfo[@"address"]];
    if(device == nil){
        reject(@"Error: ", @"Deviced is null", nil);
        return;
    }
    
    NTFirmwareVersion* version = [device version];
    NSDictionary* currentState = [NSDictionary dictionaryWithObjectsAndKeys:@(version.version), @"version", @(version.build), @"build", nil];
    resolve(currentState);
}

RCT_EXPORT_METHOD(Execute:(NSDictionary*) deviceInfo
                  command:(NSString*)command
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject)
{
    BrainBit* device = [_devices objectForKey:deviceInfo[@"address"]];
    if(device == nil){
        reject(@"Error: ", @"Deviced is null", nil);
        return;
    }
    
    if([command isEqualToString:START_SIGNAL]){
        [device startSignal];
    }
    if([command isEqualToString:STOP_SIGNAL]){
        [device stopSignal];
    }
    if([command isEqualToString:START_RESIST]){
        [device startResist];
    }
    if([command isEqualToString:STOP_RESIST]){
        [device stopResist];
    }

    resolve(@"Command executed");
}

RCT_EXPORT_METHOD(Channels:(NSDictionary*) deviceInfo
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject)
{
    BrainBit* device = [_devices objectForKey:deviceInfo[@"address"]];
    if(device == nil){
        reject(@"Error: ", @"Deviced is null", nil);
        return;
    }
    
    NSMutableArray* channelsArray = [NSMutableArray new];
    for (NTChannelInfo* info in [device channels]) {
        NSMutableDictionary<NSString*, NSString*>* convertedInfo= [NSMutableDictionary new];
        [convertedInfo setObject:info.name forKey:@"name"];
        NSString* channelName = @"";
        switch (info.type) {
            case NTChannelTypeSignal:
                channelName = SIGNAL_CHANNEL;
                break;
            case NTChannelTypeBattery:
                channelName = BATTERY_CHANNEL;
                break;
            case NTChannelTypeResistance:
                channelName = RESISTANCE_CHANNEL;
                break;
            default:
                break;
        }
        [convertedInfo setObject:channelName forKey:@"type"];
        [convertedInfo setObject:[NSString stringWithFormat:@"%d", (int)info.index] forKey:@"index"];
        [channelsArray addObject:convertedInfo];
    }
    if([channelsArray count] < 1){
        reject(@"Error: ", @"Channels not found", nil);
    } else {
        resolve(channelsArray);
    }
}

RCT_EXPORT_METHOD(Commands:(NSDictionary*) deviceInfo
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject)
{
    BrainBit* device = [_devices objectForKey:deviceInfo[@"address"]];
    if(device == nil){
        reject(@"Error: ", @"Deviced is null", nil);
        return;
    }
    
    NSMutableArray* commandsArray = [NSMutableArray new];
    for (NSNumber* command in [device commands]) {
        if(command == [NSNumber numberWithInteger:NTCommandStartSignal] ){ [commandsArray addObject:START_SIGNAL]; }
        if(command == [NSNumber numberWithInteger:NTCommandStopSignal] ){ [commandsArray addObject:STOP_SIGNAL]; }
        if(command == [NSNumber numberWithInteger:NTCommandStartResist] ){ [commandsArray addObject:START_RESIST]; }
        if(command == [NSNumber numberWithInteger:NTCommandStopResist] ){ [commandsArray addObject:STOP_RESIST]; }
    }
    if([commandsArray count] < 1){
        reject(@"Error: ", @"Commands not found", nil);
    } else {
        resolve(commandsArray);
    }
}

RCT_EXPORT_METHOD(Parameters:(NSDictionary*) deviceInfo
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject)
{
    BrainBit* device = [_devices objectForKey:deviceInfo[@"address"]];
    if(device == nil){
        reject(@"Error: ", @"Deviced is null", nil);
        return;
    }
    
    NSMutableArray* paramsArray = [NSMutableArray new];
    for (NTParameterInfo* param in [device parameters]){
        NSMutableDictionary<NSString*, NSString*>* convertedParam = [NSMutableDictionary new];
        NSString* paramName = @"";
        switch(param.parameter) {
            case NTParameterName:
                paramName = @"Name";
                break;
            case NTParameterState:
                paramName = @"State";
                break;
            case NTParameterAddress:
                paramName = @"Address";
                break;
            case NTParameterSerialNumber:
                paramName = @"SerialNumber";
                break;
            case NTParameterHardwareFilterState:
                paramName = @"HardwareFilterState";
                break;
            case NTParameterFirmwareMode:
                paramName = @"FirmwareMode";
                break;
            case NTParameterSamplingFrequency:
                paramName = @"SamplingFrequency";
                break;
            case NTParameterGain:
                paramName = @"Gain";
                break;
            case NTParameterOffset:
                paramName = @"Offset";
                break;
            case NTParameterExternalSwitchState:
                paramName = @"ExternalSwitchState";
                break;
            case NTParameterADCInputState:
                paramName = @"ADCInputState";
                break;
            case NTParameterAccelerometerSens:
                paramName = @"AccelerometerSens";
                break;
            case NTParameterGyroscopeSens:
                paramName = @"GyroscopeSens";
                break;
            case NTParameterStimulatorAndMAState:
                paramName = @"StimulatorAndMAState";
                break;
            case NTParameterStimulatorParamPack:
                paramName = @"StimulatorParamPack";
                break;
            case NTParameterMotionAssistantParamPack:
                paramName = @"MotionAssistantParamPack";
                break;
            case NTParameterFirmwareVersion:
                paramName = @"FirmwareVersion";
                break;
            case NTParameterMEMSCalibrationStatus:
                paramName = @"MEMSCalibrationStatus";
                break;
            case NTParameterNone:
                paramName = @"None";
                break;
        }
        [convertedParam setObject:paramName forKey:@"name"];
        NSString* paramAccess = @"";
        switch (param.access) {
            case NTParamAccessRead:
                paramAccess = @"Read";
                break;
            case NTParamAccessReadWrite:
                paramAccess = @"ReadWrite";
                break;
            case NTParamAccessReadNotify:
                paramAccess = @"ReadNotify";
                break;
            case NTParamAccessNone:
                paramAccess = @"None";
                break;
        }
        [convertedParam setObject:paramAccess forKey:@"access"];
        [paramsArray addObject:convertedParam];
    }
    if([paramsArray count] < 1){
        reject(@"Error: ", @"Commands not found", nil);
    } else {
        resolve(paramsArray);
    }
}

RCT_EXPORT_METHOD(Disconnect:(NSDictionary*) deviceInfo
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject)
{
    BrainBit* device = [_devices objectForKey:deviceInfo[@"address"]];
    if(device == nil){
        reject(@"Error: ", @"Deviced is null", nil);
        return;
    }
    [device disconnect];
    NSDictionary* currentState = [NSDictionary dictionaryWithObjectsAndKeys:@"Disconnection", @"state", nil];
    resolve(currentState);
}

RCT_EXPORT_METHOD(Close:(NSDictionary*) deviceInfo
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject)
{
    BrainBit* device = [_devices objectForKey:deviceInfo[@"address"]];
    [_devices removeObjectForKey:deviceInfo[@"address"]];
    if(device == nil){
        reject(@"Error: ", @"Deviced is null", nil);
        return;
    }
    [device close];
    device = nil;
    
    NSDictionary* currentState = [NSDictionary dictionaryWithObjectsAndKeys:@"Closed", @"state", nil];
    resolve(currentState);
}

/* Channels */
RCT_EXPORT_METHOD(CreateChannel:(NSDictionary*) deviceInfo
                  channelInfo:(NSDictionary*) channelInfo
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject)
{
    BrainBit* device = [_devices objectForKey:deviceInfo[@"address"]];
    if(device == nil){
        reject(@"Error: ", @"Deviced is null", nil);
        return;
    }
    NSString* type = [channelInfo objectForKey:@"type"];
    if([type isEqualToString:SIGNAL_CHANNEL]){
        [device initSignalWithName:channelInfo[@"name"]];
    } else if([type isEqualToString:RESISTANCE_CHANNEL]){
        [device initResistanceChannelWithName:channelInfo[@"name"]];
    }else if([type isEqualToString:BATTERY_CHANNEL]){
        [device initBattery];
    } else
    {
        reject(@"Error", @"Channel type not defined", nil);
        return;
    }
    resolve(@"Channel created");
}

RCT_EXPORT_BLOCKING_SYNCHRONOUS_METHOD(TotalLengthSyncChannel:(NSDictionary*) deviceInfo
                                       channelInfo:(NSDictionary*) channelInfo){
    NSUInteger length = -1;
    BrainBit* device = [_devices objectForKey:deviceInfo[@"address"]];
    if(device == nil){
        length = -1;
    }
    if([channelInfo[@"type"] isEqualToString:SIGNAL_CHANNEL]){
        length = [device signalTotalLengthOfChannel:channelInfo[@"name"]];
    } else if([channelInfo[@"type"] isEqualToString:RESISTANCE_CHANNEL]){
        length = [device resistanceTotalLengthOfChannel:channelInfo[@"name"]];
    } else if([channelInfo[@"type"] isEqualToString:BATTERY_CHANNEL]){
        length = [device batteryTotalLength];
    } else {
        length = -1;
    }
    NSNumber* numberLength = [NSNumber numberWithInteger:length];
    NSDictionary* result = [NSDictionary dictionaryWithObjectsAndKeys:numberLength, @"totalLength", nil];
    return result;
}

RCT_EXPORT_METHOD(TotalLengthChannel:(NSDictionary*) deviceInfo
                  channelInfo:(NSDictionary*) channelInfo
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject)
{
    BrainBit* device = [_devices objectForKey:deviceInfo[@"address"]];
    if(device == nil){
        reject(@"Error: ", @"Deviced is null", nil);
    }
    NSUInteger length = 0;
    if([channelInfo[@"type"] isEqualToString:SIGNAL_CHANNEL]){
        length = [device signalTotalLengthOfChannel:channelInfo[@"name"]];
    } else if([channelInfo[@"type"] isEqualToString:RESISTANCE_CHANNEL]){
        length = [device resistanceTotalLengthOfChannel:channelInfo[@"name"]];
    } else if([channelInfo[@"type"] isEqualToString:BATTERY_CHANNEL]){
        length = [device batteryTotalLength];
    } else {
        reject(@"Error: ", @"Channel type not recognized", nil);
    }
        NSNumber* numberLength = [NSNumber numberWithInteger:length];
        NSDictionary* result = [NSDictionary dictionaryWithObjectsAndKeys:numberLength, @"totalLength", nil];
        resolve(result);
}

RCT_EXPORT_METHOD(ReadDataChannel:(NSDictionary*) deviceInfo
                  channelInfo:(NSDictionary*) channelInfo
                  offset:(NSNumber* _Nonnull) offset
                  length:(NSNumber* _Nonnull) length
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject)
{
    BrainBit* device = [_devices objectForKey:deviceInfo[@"address"]];
    if(device == nil){
        reject(@"Error: ", @"Deviced is null", nil);
    }
    NSArray<NSNumber*>* data = [NSArray new];
    if([channelInfo[@"type"] isEqualToString:SIGNAL_CHANNEL]){
        data = [device readSignalDataFromChannel:channelInfo[@"name"] offset:[offset unsignedLongValue] length:[length unsignedLongValue]];
    } else if([channelInfo[@"type"] isEqualToString:RESISTANCE_CHANNEL]){
        data = [device readResistanceDataFromChannel:channelInfo[@"name"] offset:[offset unsignedLongValue] length:[length unsignedLongValue]];
    } else if([channelInfo[@"type"] isEqualToString:BATTERY_CHANNEL]){
        data = [device readBatteryData:[offset unsignedLongValue] length:[length unsignedLongValue]];
    } else {
        reject(@"Error: ", @"Channel type not recognized", nil);
    }
    resolve(data);
}

RCT_EXPORT_BLOCKING_SYNCHRONOUS_METHOD(ReadDataSyncChannel:(NSDictionary*) deviceInfo
                  channelInfo:(NSDictionary*) channelInfo
                  offset:(NSNumber* _Nonnull) offset
                  length:(NSNumber* _Nonnull) length)
{
    BrainBit* device = [_devices objectForKey:deviceInfo[@"address"]];
    if(device == nil){
        return nil;
    }
    NSArray<NSNumber*>* data = [NSArray new];
    if([channelInfo[@"type"] isEqualToString:SIGNAL_CHANNEL]){
        data = [device readSignalDataFromChannel:channelInfo[@"name"] offset:[offset unsignedLongValue] length:[length unsignedLongValue]];
    } else if([channelInfo[@"type"] isEqualToString:RESISTANCE_CHANNEL]){
        data = [device readResistanceDataFromChannel:channelInfo[@"name"] offset:[offset unsignedLongValue] length:[length unsignedLongValue]];
    } else if([channelInfo[@"type"] isEqualToString:BATTERY_CHANNEL]){
        data = [device readBatteryData:[offset unsignedLongValue] length:[length unsignedLongValue]];
    } else {
        return nil;
    }
    return data;
}

RCT_EXPORT_METHOD(CloseChannel:(NSDictionary*) deviceInfo
                  channelInfo:(NSDictionary*) channelInfo
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject)
{
    BrainBit* device = [_devices objectForKey:deviceInfo[@"address"]];
    if(device == nil){
        reject(@"Error: ", @"Deviced is null", nil);
        return;
    }
    if([channelInfo[@"type"] isEqualToString:SIGNAL_CHANNEL]){
        [device destroySignalWithName:channelInfo[@"name"]];
    } else if([channelInfo[@"type"] isEqualToString:RESISTANCE_CHANNEL]){
        [device destroyResistanceWithName:channelInfo[@"name"]];
    } else if([channelInfo[@"type"] isEqualToString:BATTERY_CHANNEL]){
        [device destroyBattery];
    } else {
        reject(@"Error: ", @"Channel type not recognized", nil);
        return;
    }
    resolve(@"Channel destroyed");
}

@end
  
